@@ -8,7 +8,7 @@
 #
 package PerlIO::via::Timeout::Strategy::NoTimeout;
 {
-  $PerlIO::via::Timeout::Strategy::NoTimeout::VERSION = '0.19';
+  $PerlIO::via::Timeout::Strategy::NoTimeout::VERSION = '0.20';
 }
 
 # ABSTRACT: a L<PerlIO::via::Timeout> strategy that don't do any timeout
@@ -17,21 +17,31 @@ package PerlIO::via::Timeout::Strategy::NoTimeout;
 require 5.008;
 use strict;
 use warnings;
+use Errno qw(EINTR);
 
 use PerlIO::via::Timeout::Strategy;
 our @ISA = qw(PerlIO::via::Timeout::Strategy);
 
+
 sub READ {
     my ($self, undef, $len, $fh, $fd) = @_;
-    my $rv = sysread($fh, $_[1], $len);
-    if (! defined $rv) {
-        # There is a bug in PerlIO::via (possibly in PerlIO ?). We would like
-        # to return -1 to signify error, but doing so doesn't work (it usually
-        # segfault), it looks like the implementation is not complete. So we
-        # return 0.
-        $rv = 0;
+    my $offset = 0;
+    while () {
+        my $r = sysread($fh, $_[1], $len, $offset);
+        if (defined $r) {
+            last unless $r;
+            $len -= $r;
+            $offset += $r;
+        }
+        elsif ($! != EINTR) {
+            # There is a bug in PerlIO::via (possibly in PerlIO ?). We would like
+            # to return -1 to signify error, but doing so doesn't work (it usually
+            # segfault), it looks like the implementation is not complete. So we
+            # return 0.
+            return 0;
+        }
     }
-    return $rv;
+    return $offset;
 }
 
 sub WRITE {
@@ -53,7 +63,7 @@ PerlIO::via::Timeout::Strategy::NoTimeout - a L<PerlIO::via::Timeout> strategy t
 
 =head1 VERSION
 
-version 0.19
+version 0.20
 
 =DESCRIPTION
 
